(function() {
  var Bridge, CSI, check, dataFile, fsEnabled, getBackgroundColor, getBackgroundHex, getThemeClass, menuEvent, menuXML, migrateLocalStorage, onAppThemeColorChanged, onDocumentLoaded, onMenuClick, persistent, ready, themeChangeEvent, tick, updateSessionInfo;

  CSI = new CSInterface();

  Bridge = null;

  themeChangeEvent = CSInterface.THEME_COLOR_CHANGED_EVENT;

  menuEvent = "com.adobe.csxs.events.flyoutMenuClicked";

  menuXML = function() {
    return "<Menu>\n  <MenuItem Id=\"version\" Label=\"GuideGuide " + (Bridge.guideguideVersion()) + (Bridge.freeVersion() ? ' (free)' : '') + "\" Enabled=\"false\" Checked=\"false\"/>\n  <MenuItem Label=\"---\" />\n  <MenuItem Id=\"tracking\" Label=\"Tracking settings\" Enabled=\"true\" Checked=\"false\"/>\n  <MenuItem Id=\"uninstall\" Label=\"Uninstall\" Enabled=\"true\" Checked=\"false\"/>\n  " + (Bridge.freeVersion() ? '<MenuItem Id=\"buy\" Label=\"Buy GuideGuide\" Enabled=\"true\" Checked=\"false\"/>' : '') + "\n</Menu>";
  };

  persistent = function() {
    var csInterface, event, id;
    event = new CSEvent('com.adobe.PhotoshopPersistent', 'APPLICATION');
    id = "com.cameronmcefee.guideguide";
    event.extensionId = id;
    csInterface = new CSInterface();
    return csInterface.dispatchEvent(event);
  };

  updateSessionInfo = function() {
    return Bridge.fetchAndSyncDocumentInfo();
  };

  getThemeClass = function() {
    var color;
    color = getBackgroundColor();
    if (color._r < 64) {
      return 'theme-darker';
    }
    if (color._r < 128) {
      return 'theme-dark';
    }
    if (color._r < 192) {
      return 'theme-light';
    }
    return 'theme-lighter';
  };

  getBackgroundColor = function() {
    var color;
    color = JSON.parse(window.__adobe_cep__.getHostEnvironment()).appSkinInfo.panelBackgroundColor.color;
    return tinycolor("rgba(" + color.red + "," + color.green + "," + color.blue + "," + color.alpha + ")");
  };

  getBackgroundHex = function() {
    return getBackgroundColor().toHexString();
  };

  fsEnabled = function() {
    var fs;
    fs = window.cep.fs;
    if (fs == null) {
      return false;
    }
    if (fs.showOpenDialog == null) {
      return false;
    }
    if (fs.readFile == null) {
      return false;
    }
    if (fs.showSaveDialogEx == null) {
      return false;
    }
    if (fs.writeFile == null) {
      return false;
    }
    return true;
  };

  check = function() {
    Bridge = document.getElementById('panel').contentWindow.Bridge;
    if (Bridge && Bridge.ready) {
      return ready();
    } else {
      return tick();
    }
  };

  tick = function() {
    return setTimeout(check, 50);
  };

  dataFile = function() {
    var file, folder, result;
    folder = "" + (CSI.getSystemPath(SystemPath.MY_DOCUMENTS)) + "/GuideGuide";
    file = "" + folder + "/data.json";
    result = window.cep.fs.readFile(file);
    if (result.err === 0) {
      return file;
    }
    result = window.cep.fs.makedir(folder);
    if (result.err !== 0) {
      return null;
    }
    result = window.cep.fs.writeFile(file, JSON.stringify({}));
    if (result.err !== 0) {
      return null;
    }
    return file;
  };

  migrateLocalStorage = function() {
    var i, k, ls, v, _i, _j, _len, _ref, _ref1, _results, _results1;
    ls = document.getElementById('panel').contentWindow.localStorage;
    if (!ls.length) {
      return;
    }
    Bridge.log('Migrating localStorage values');
    _ref1 = (function() {
      _results1 = [];
      for (var _j = 0, _ref = ls.length - 1; 0 <= _ref ? _j <= _ref : _j >= _ref; 0 <= _ref ? _j++ : _j--){ _results1.push(_j); }
      return _results1;
    }).apply(this).reverse();
    _results = [];
    for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
      i = _ref1[_i];
      k = ls.key(i);
      v = ls.getItem(k);
      Bridge.localStorage().setItem(k, v);
      ls.removeItem(k);
      _results.push(Bridge.log("Migrated " + k));
    }
    return _results;
  };

  ready = function() {
    var dataPath, _ref;
    if (window.__adobe_cep__.invokeSync != null) {
      CSI.setPanelFlyoutMenu(menuXML());
    }
    CSI.addEventListener('documentAfterActivate', updateSessionInfo);
    CSI.addEventListener('documentAfterDeactivate', updateSessionInfo);
    CSI.addEventListener(themeChangeEvent, onAppThemeColorChanged);
    CSI.addEventListener(menuEvent, onMenuClick);
    dataPath = dataFile();
    if ((((_ref = window.cep) != null ? _ref.fs : void 0) != null) && (dataPath != null)) {
      Bridge.localStorage = function() {
        return {
          getItem: function(k) {
            return JSON.parse(window.cep.fs.readFile(dataPath).data)[k];
          },
          setItem: function(k, v) {
            var obj;
            obj = JSON.parse(window.cep.fs.readFile(dataPath).data);
            obj[k] = v;
            return window.cep.fs.writeFile(dataPath, JSON.stringify(obj));
          },
          removeItem: function(k, v) {
            var obj;
            obj = JSON.parse(window.cep.fs.readFile(dataPath).data);
            delete obj[k];
            return window.cep.fs.writeFile(dataPath, JSON.stringify(obj));
          }
        };
      };
      migrateLocalStorage();
    }
    Bridge.exec = function(method, data, callback) {
      var methodStr;
      if (typeof arguments[1] === 'function') {
        callback = arguments[1];
        data = null;
      }
      methodStr = "$.GuideGuide." + method + "(" + (data ? '\'' + JSON.stringify(data) + '\'' : '') + ")";
      return CSI.evalScript(methodStr, function(result) {
        var json;
        json = typeof result === 'string' ? JSON.parse(result) : null;
        if (callback != null) {
          return callback(null, json);
        }
      });
    };
    Bridge.getLocale = function() {
      var env;
      env = JSON.parse(window.__adobe_cep__.getHostEnvironment());
      return env.appUILocale.toLowerCase();
    };
    Bridge.getAppVersion = function() {
      return "" + (CSI.getApplicationID()) + " " + (parseInt(CSI.getHostEnvironment().appVersion));
    };
    Bridge.fetchAndSyncAppInfo = function(callback) {
      var _this = this;
      return this.exec('getAppInfo', function(err, info) {
        info.themeClass = getThemeClass();
        info.locale = _this.getLocale();
        info.localStorage = true;
        info.fsEnabled = fsEnabled();
        _this.syncAppInfo(info);
        if (callback != null) {
          return callback();
        }
      });
    };
    Bridge.getDocumentRulerUnits = function(callback) {
      return CSI.evalScript('$.GuideGuide.getDocumentRulerUnits()', callback);
    };
    Bridge.openUrl = function(url) {
      return CSI.openURLInDefaultBrowser(url);
    };
    Bridge.openInstallDirectory = function() {
      var path;
      path = CSI.getSystemPath(SystemPath.EXTENSION);
      path = path.substring(0, path.lastIndexOf('/'));
      return CSI.evalScript("$.GuideGuide.openInstallDirectory('" + path + "')");
    };
    Bridge.readFile = function(callback) {
      var file, result;
      result = window.cep.fs.showOpenDialog(false, false, 'open', null, ['json']);
      if (result.err) {
        return callback('fileError');
      }
      if (!/\.json$/gi.test(result.data[0])) {
        return callback('notJSON');
      }
      file = window.cep.fs.readFile(result.data[0]);
      if (file.err) {
        return callback('fileError');
      }
      return callback(null, file.data);
    };
    Bridge.writeFile = function(filename, data, callback) {
      var result, save, show, title;
      show = window.cep.fs.showSaveDialogEx;
      title = this.Messages.get('titleExportGrids');
      save = this.Messages.get('btnExport');
      result = show(title, null, ['json'], 'grids.json', null, save);
      if (result.data === '') {
        return callback('canceled');
      }
      result = window.cep.fs.writeFile(result.data, data);
      if (result.err) {
        return callback(result.err);
      }
      if (callback != null) {
        return callback();
      }
    };
    return Bridge.fetchAndSyncAppInfo(function() {
      return Bridge.fetchAndSyncDocumentInfo(function() {
        Bridge.updateTheme();
        Bridge.start();
        document.getElementById('panel').style.opacity = 1;
        return persistent();
      });
    });
  };

  onAppThemeColorChanged = function(event) {
    Bridge.syncAppInfo({
      themeClass: getThemeClass()
    });
    return Bridge.updateTheme();
  };

  onMenuClick = function(event) {
    switch (event.data.menuId) {
      case 'tracking':
        return Bridge.changeTrackingSettings();
      case 'debug':
        return Bridge.debug();
      case 'uninstall':
        return Bridge.openInstallDirectory();
      case 'buy':
        return Bridge.openUrl('http://guideguide.me/');
    }
  };

  onDocumentLoaded = function() {
    var iframe;
    document.body.style.backgroundColor = getBackgroundHex();
    iframe = document.getElementById('panel');
    iframe.src = iframe.getAttribute('data-src');
    return tick();
  };

  document.addEventListener('DOMContentLoaded', onDocumentLoaded, false);

}).call(this);
